@@ -51,6 +51,7 @@
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-stock.h>
+#include <libgames-support/games-sound.h>
 
 #include "board.h"
 #include "conf.h"
@@ -87,6 +88,7 @@ enum
   ACTION_FULLSCREEN,
   ACTION_HELP_GAME,
   ACTION_OPTIONS_MENU,
+  ACTION_DEAL,
   ACTION_HINT,
 #ifdef HAVE_MAEMO
   ACTION_ACCEL_UNDO_MOVE,
@@ -143,8 +145,6 @@ struct _AisleriotWindowPrivate
   guint scalable_cards : 1;
   guint use_pixbuf_drawing : 1;
   guint changing_game_type : 1;
-  guint is_maximised : 1;
-  guint is_fullscreen : 1;
   guint freecell_mode : 1;
   guint toolbar_visible : 1;
 };
@@ -226,7 +226,7 @@ select_game_cb (GtkAction *action,
   list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
   g_object_unref (list);
 
-  files = games_file_list_new ("*.scm", GAMESDIR, NULL);
+  files = games_file_list_new ("*.scm", games_path_runtime_fix (GAMESDIR), NULL);
   games_file_list_transform_basename (files);
 
   current_game_file = aisleriot_game_get_game_file (priv->game);
@@ -404,8 +404,11 @@ show_game_over_dialog (AisleriotWindow *window)
 
   if (game_won) {
     message = _("Congratulations, you have won!");
+    games_sound_play ("victory");
+
   } else {
     message =  _("There are no more moves");
+    games_sound_play ("splat");
   }
 
   dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (window),
@@ -550,6 +553,7 @@ help_about_cb (GtkAction *action,
     "Rosanna Yuen <zana@webwynk.net>",
     "Callum McKenzie <callum@physics.otago.ac.nz>",
     "Christian Persch <chpe" "\100" "gnome.org>",
+    "Andreas Røsdal <andreasr" "\100" "gnome.org>",
     "",
     _("Card games:"),
     "Jonathan Blandford <jrb@redhat.com>",
@@ -559,6 +563,8 @@ help_about_cb (GtkAction *action,
     "Changwoo Ryu <cwryu@adam.kaist.ac.kr>",
     "Matthew Wilcox <matthew@wil.cx>",
     "Rosanna Yuen <zana@webwynk.net>",
+    "Alan Horkan <horkana@maths.tcd.ie>",
+    "Richard Hoelscher <rah@rahga.com>",
     NULL
   };
   const char *artists[] = {
@@ -569,6 +575,8 @@ help_about_cb (GtkAction *action,
     "Nicu Buculei http://www.nicubunu.ro/cards",
     /* Dondorf and Paris cards */
     "Richard Hoelscher http://www.rahga.com/svg",
+    /* Anglo-American cards */
+    "Aike Reyer",
     /* FIXMEchpe: who did the Bonded cards? */
     NULL
   };
@@ -599,7 +607,8 @@ help_about_cb (GtkAction *action,
                            NULL :
                            _("AisleRiot provides a rule-based solitaire "
                              "card engine that allows many different "
-                             "games to be played."),
+                             "games to be played.\n\n"
+			     "AisleRiot is a part of GNOME Games."),
 #endif
                          "copyright", "Copyright © 1998-2006 Jonathan Blandford\n"
                                       "Copyright © 2007 Christian Persch",
@@ -760,7 +769,7 @@ debug_cycle_cb (GtkAction *action,
   GamesFileList *files;
   DebugCycleData *data;
 
-  files = games_file_list_new ("*.scm", GAMESDIR, NULL);
+  files = games_file_list_new ("*.scm", games_path_runtime_fix (GAMESDIR), NULL);
   games_file_list_transform_basename (files);
 
   data = g_slice_new (DebugCycleData);
@@ -855,7 +864,7 @@ toolbar_toggled_cb (GtkToggleAction *action,
 
   priv->toolbar_visible = state != FALSE;
 
-  aisleriot_conf_set_boolean (CONF_SHOW_TOOLBAR, state);
+  games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_SHOW_TOOLBAR), state);
 }
 
 static void
@@ -908,7 +917,21 @@ clickmove_toggle_cb (GtkToggleAction *action,
   aisleriot_game_set_click_to_move (priv->game, click_to_move);
   aisleriot_board_set_click_to_move (priv->board, click_to_move);
   
-  aisleriot_conf_set_boolean (CONF_CLICK_TO_MOVE, click_to_move);
+  games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_CLICK_TO_MOVE), click_to_move);
+}
+
+static void
+sound_toggle_cb (GtkToggleAction *action,
+                 AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  gboolean sound_enabled;
+
+  sound_enabled = gtk_toggle_action_get_active (action);
+
+  games_sound_enable (sound_enabled);
+  
+  games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_SOUND), sound_enabled);
 }
 
 static void
@@ -1063,6 +1086,15 @@ show_hint_cb (GtkAction *action,
   g_free (message);
 }
 
+static void
+deal_cb (GtkAction *action,
+         AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  aisleriot_game_deal_cards (priv->game);
+}
+
 /* Make something that's easier to store in conf */
 static guint
 compress_options_to_int (SCM options_list)
@@ -1288,7 +1320,7 @@ add_recently_played_game (AisleriotWindow *window,
   if (priv->freecell_mode)
     return;
 
-  recent_games = aisleriot_conf_get_strings (CONF_RECENT_GAMES, &n_recent);
+  recent_games = games_conf_get_string_list (NULL, aisleriot_conf_get_key (CONF_RECENT_GAMES), &n_recent, NULL);
 
   if (recent_games == NULL) {
     new_recent = g_new (char *, 2);
@@ -1313,7 +1345,8 @@ add_recently_played_game (AisleriotWindow *window,
     g_strfreev (recent_games);
   }
 
-  aisleriot_conf_set_strings (CONF_RECENT_GAMES, (const char * const *) new_recent, n_new_recent);
+  games_conf_set_string_list (NULL, aisleriot_conf_get_key (CONF_RECENT_GAMES),
+                              (const char * const *) new_recent, n_new_recent);
   g_strfreev (new_recent);
 }
 
@@ -1328,7 +1361,7 @@ recent_game_cb (GtkAction *action,
 
   aisleriot_window_set_game (window, game_file, 0);
   
-  aisleriot_conf_set_string (CONF_VARIATION, game_file);
+  games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_VARIATION), game_file);
 }
 
 static void
@@ -1355,7 +1388,7 @@ install_recently_played_menu (AisleriotWindow *window)
 
   priv->recent_games_merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
-  recent_games = aisleriot_conf_get_strings (CONF_RECENT_GAMES, &n_recent);
+  recent_games = games_conf_get_string_list (NULL, aisleriot_conf_get_key (CONF_RECENT_GAMES), &n_recent, NULL);
 
   for (i = 0; i < n_recent; ++i) {
     GtkAction *action;
@@ -1416,7 +1449,7 @@ card_theme_changed_cb (GtkToggleAction *action,
 #endif /* HAVE_GNOME */
 
   if (aisleriot_board_set_card_theme (priv->board, theme)) {
-    aisleriot_conf_set_string (CONF_THEME, theme);
+    games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_THEME), theme);
   }
 }
 
@@ -1528,25 +1561,6 @@ sync_window_topmost_cb (AisleriotWindow *window,
 
 #endif /* HAVE_MAEMO */
 
-#ifndef HAVE_MAEMO
-
-static void
-sync_game_features (AisleriotGame *game,
-                    GParamSpec *pspec,
-                    AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  guint features = 0;
-  gboolean show;
-
-  g_object_get (game, "features", &features, NULL);
-
-  show = (features & FEATURE_SCORE_HIDDEN) == 0;
-  g_object_set (priv->score_box, "visible", show, NULL);
-}
-
-#endif /* !HAVE_MAEMO */
-
 /* Game state synchronisation */
 
 #ifndef HAVE_MAEMO
@@ -1640,12 +1654,30 @@ sync_game_redoable (AisleriotGame *game,
   gtk_action_set_sensitive (priv->action[ACTION_REDO_MOVE], enabled);
 }
 
+static void
+sync_game_dealable (AisleriotGame *game,
+                    GParamSpec *pspec,
+                    AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  gboolean enabled;
+
+  g_object_get (game, "can-deal", &enabled, NULL);
+
+  gtk_action_set_sensitive (priv->action[ACTION_DEAL], enabled);
+}
+
 static void
 game_type_changed_cb (AisleriotGame *game,
                       AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
   char *game_name;
+  guint features;
+  gboolean dealable;
+#ifndef HAVE_MAEMO
+  gboolean show_scores;
+#endif /* !HAVE_MAEMO */
 
   priv->changing_game_type = TRUE;
 
@@ -1666,12 +1698,20 @@ game_type_changed_cb (AisleriotGame *game,
 
   update_statistics_display (window);
 
+  features = aisleriot_game_get_features (game);
+
+  dealable = (features & FEATURE_DEALABLE) != 0;
+  gtk_action_set_visible (priv->action[ACTION_DEAL], dealable);
+
 #ifdef HAVE_MAEMO
   gtk_label_set_text (priv->game_message_label, NULL);
 #else
   games_clock_set_seconds (GAMES_CLOCK (priv->clock), 0);
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
+
+  show_scores = (features & FEATURE_SCORE_HIDDEN) == 0;
+  g_object_set (priv->score_box, "visible", show_scores, NULL);
 #endif
 
   priv->changing_game_type = FALSE;
@@ -1835,27 +1875,6 @@ G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, HILDON_TYPE_WINDOW);
 G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, GTK_TYPE_WINDOW);
 #endif
 
-#ifndef HAVE_MAEMO
-
-/* FIXMEchpe maybe use size-allocate instead? */
-static gboolean
-aisleriot_window_configure (GtkWidget *widget,
-                            GdkEventConfigure *event)
-{
-  AisleriotWindow *window = AISLERIOT_WINDOW (widget);
-  AisleriotWindowPrivate *priv = window->priv;
-
-  if (!priv->is_maximised && !priv->is_fullscreen) {
-    /* FIXMEchpe: not on every configure event !!! do it on idle afterwards */
-    aisleriot_conf_set_int (CONF_WIDTH, event->width);
-    aisleriot_conf_set_int (CONF_HEIGHT, event->height);
-  }
-
-  return GTK_WIDGET_CLASS (aisleriot_window_parent_class)->configure_event (widget, event);
-}
-
-#endif /* !HAVE_MAEMO */
-
 static gboolean
 aisleriot_window_state_event (GtkWidget *widget,
                               GdkEventWindowState *event)
@@ -1864,21 +1883,20 @@ aisleriot_window_state_event (GtkWidget *widget,
   AisleriotWindowPrivate *priv = window->priv;
 
   if (event->changed_mask & (GDK_WINDOW_STATE_FULLSCREEN | GDK_WINDOW_STATE_MAXIMIZED)) {
-    priv->is_fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
-    priv->is_maximised = (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) != 0;
+    gboolean is_fullscreen, is_maximised;
 
-    set_fullscreen_actions (window, priv->is_fullscreen);
+    is_fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
+    is_maximised = (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) != 0;
+
+    set_fullscreen_actions (window, is_fullscreen);
 
 #ifndef HAVE_MAEMO
 #if GTK_CHECK_VERSION (2, 11, 0)
-    gtk_statusbar_set_has_resize_grip (priv->statusbar, !priv->is_maximised && !priv->is_fullscreen);
+    gtk_statusbar_set_has_resize_grip (priv->statusbar, !is_maximised && !is_fullscreen);
 #else
     gtk_statusbar_set_has_resize_grip (priv->statusbar, FALSE);
 #endif
 #endif /* !HAVE_MAEMO */
-
-    aisleriot_conf_set_boolean (CONF_SHOW_MAXIMISED, priv->is_maximised);
-    aisleriot_conf_set_boolean (CONF_SHOW_FULLSCREEN, priv->is_fullscreen);
   }
 
 #ifndef HAVE_MAEMO
@@ -1946,6 +1964,9 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (undo_cb) },
     { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, NULL,
       G_CALLBACK (redo_cb) },
+    { "Deal", GTK_STOCK_OK /* FIXMEchpe */, _("_Deal"), "<control>D",
+      ACTION_TOOLTIP (N_("Deal next card or cards")),
+      G_CALLBACK (deal_cb) },
     { "Hint", GAMES_STOCK_HINT, NULL, NULL, NULL,
       G_CALLBACK (show_hint_cb) },
     { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL,
@@ -2002,6 +2023,10 @@ aisleriot_window_init (AisleriotWindow *window)
       ACTION_TOOLTIP (N_("Pick up and drop cards by clicking")),
       G_CALLBACK (clickmove_toggle_cb),
       FALSE /* not active by default */ },
+   { "Sound", NULL, N_("_Enable sounds"), NULL,
+      ACTION_TOOLTIP (N_("Whether or not to play event sounds.")),
+      G_CALLBACK (sound_toggle_cb),
+      FALSE /* not active by default */ },
 #ifdef ENABLE_DEBUG_UI
     { "DebugPixbufDrawing", NULL, "_Pixbuf drawing", NULL, NULL,
       G_CALLBACK (debug_pixbuf_drawing_cb),
@@ -2016,6 +2041,7 @@ aisleriot_window_init (AisleriotWindow *window)
     "Fullscreen",
     "HelpGame",
     "OptionsMenu",
+    "Deal",
     "Hint",
 #ifdef HAVE_MAEMO
     "AccelUndoMove",
@@ -2047,6 +2073,7 @@ aisleriot_window_init (AisleriotWindow *window)
         "<menu action='ControlMenu'>"
           "<menuitem action='UndoMove'/>"
           "<menuitem action='RedoMove'/>"
+          "<menuitem action='Deal'/>"
           "<menuitem action='Hint'/>"
           "<separator/>"
           "<menuitem action='ClickToMove'/>"
@@ -2086,9 +2113,11 @@ aisleriot_window_init (AisleriotWindow *window)
         "<menu action='ControlMenu'>"
           "<menuitem action='UndoMove'/>"
           "<menuitem action='RedoMove'/>"
+          "<menuitem action='Deal'/>"
           "<menuitem action='Hint'/>"
           "<separator/>"
           "<menuitem action='ClickToMove'/>"
+          "<menuitem action='Sound'/>"
         "</menu>"
         "<menu action='OptionsMenu'/>"
         "<menu action='HelpMenu'>"
@@ -2116,6 +2145,7 @@ aisleriot_window_init (AisleriotWindow *window)
         "<separator/>"
         "<toolitem action='UndoMove'/>"
         "<toolitem action='RedoMove'/>"
+        "<toolitem action='Deal'/>"
         "<toolitem action='Hint'/>"
 #ifndef HAVE_MAEMO
         "<toolitem action='LeaveFullscreen'/>"
@@ -2130,7 +2160,6 @@ aisleriot_window_init (AisleriotWindow *window)
     "</ui>";
 
   AisleriotWindowPrivate *priv;
-  gint width, height;
   GtkWidget *main_vbox;
   GtkAccelGroup *accel_group;
   GtkAction *action;
@@ -2179,7 +2208,7 @@ aisleriot_window_init (AisleriotWindow *window)
 
   aisleriot_board_set_pixbuf_drawing (priv->board, priv->use_pixbuf_drawing);
 
-  theme = aisleriot_conf_get_string (CONF_THEME, GAMES_CARD_THEME_DEFAULT);
+  theme = games_conf_get_string_with_default (NULL, aisleriot_conf_get_key (CONF_THEME), GAMES_CARD_THEME_DEFAULT);
 #ifdef HAVE_GNOME
   /* Compatibility with old settings */
   if (g_str_has_suffix (theme, ".svg")) {
@@ -2212,6 +2241,9 @@ aisleriot_window_init (AisleriotWindow *window)
     g_assert (priv->action[i]);
   }
 
+  /* Hide the "Deal" action initially, since not all games support it */
+  gtk_action_set_visible (priv->action[ACTION_DEAL], FALSE);
+
 #ifndef HAVE_MAEMO
   statusbar = priv->statusbar = GTK_STATUSBAR (gtk_statusbar_new ());
   priv->game_message_id = gtk_statusbar_get_context_id (priv->statusbar, "game-message");
@@ -2238,11 +2270,12 @@ aisleriot_window_init (AisleriotWindow *window)
   /* Score */
   priv->score_box = gtk_hbox_new (12, FALSE);
   label = gtk_label_new (_("Score:"));
+  gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (priv->score_box), label, FALSE, FALSE, 0);
   priv->score_label = gtk_label_new ("   0");
+  gtk_widget_show (priv->score_label);
   gtk_box_pack_start (GTK_BOX (priv->score_box), priv->score_label, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (statusbar_hbox), priv->score_box, FALSE, FALSE, 0);
-  gtk_widget_show_all (priv->score_box);
 
   time_box = gtk_hbox_new (12, FALSE);
   label = gtk_label_new (_("Time:"));
@@ -2288,12 +2321,16 @@ aisleriot_window_init (AisleriotWindow *window)
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
   action = gtk_action_group_get_action (priv->action_group, "Toolbar");
-  priv->toolbar_visible = aisleriot_conf_get_boolean (CONF_SHOW_TOOLBAR) != FALSE;
+  priv->toolbar_visible = games_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_SHOW_TOOLBAR), NULL) != FALSE;
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
                                 priv->toolbar_visible);
   action = gtk_action_group_get_action (priv->action_group, "ClickToMove");
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                                aisleriot_conf_get_boolean (CONF_CLICK_TO_MOVE));
+                                games_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_CLICK_TO_MOVE), NULL));
+  action = gtk_action_group_get_action (priv->action_group, "Sound");
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+                                games_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_SOUND), NULL));
+  gtk_action_set_visible (action, games_sound_is_available ());
   action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
   g_object_set (action, "hide-if-empty", FALSE, NULL);
 
@@ -2319,9 +2356,6 @@ aisleriot_window_init (AisleriotWindow *window)
 
   /* Synchronise */
 #ifndef HAVE_MAEMO
-  sync_game_features (priv->game, NULL, window);
-  g_signal_connect (priv->game, "notify::features",
-                    G_CALLBACK (sync_game_features), window);
   sync_game_score (priv->game, NULL, window);
   g_signal_connect (priv->game, "notify::score",
                     G_CALLBACK (sync_game_score), window);
@@ -2336,6 +2370,9 @@ aisleriot_window_init (AisleriotWindow *window)
   sync_game_redoable (priv->game, NULL, window);
   g_signal_connect (priv->game, "notify::can-redo",
                     G_CALLBACK (sync_game_redoable), window);
+  sync_game_dealable (priv->game, NULL, window);
+  g_signal_connect (priv->game, "notify::can-deal",
+                    G_CALLBACK (sync_game_dealable), window);
 
   g_signal_connect (priv->game, "game-type",
                     G_CALLBACK (game_type_changed_cb), window);
@@ -2346,22 +2383,11 @@ aisleriot_window_init (AisleriotWindow *window)
   g_signal_connect (priv->game, "exception",
                     G_CALLBACK (game_exception_cb), window);
 
+  /* Fallback, if there is no saved size */
+  gtk_window_set_default_size (GTK_WINDOW (window), MIN_WIDTH, MIN_HEIGHT);
+
   /* Restore window state */
-  width = aisleriot_conf_get_int (CONF_WIDTH);
-  height = aisleriot_conf_get_int (CONF_HEIGHT);
-
-  if (width > 0 && height > 0) {
-    gtk_window_set_default_size (GTK_WINDOW (window), width, height);
-  } else {
-    gtk_window_set_default_size (GTK_WINDOW (window), MIN_WIDTH, MIN_HEIGHT);
-  }
-
-  if (aisleriot_conf_get_boolean (CONF_SHOW_MAXIMISED)) {
-    gtk_window_maximize (GTK_WINDOW (window));
-  }
-  if (aisleriot_conf_get_boolean (CONF_SHOW_FULLSCREEN)) {
-    gtk_window_fullscreen (GTK_WINDOW (window));
-  }
+  games_conf_add_window (GTK_WINDOW (window), NULL);
 
   /* Initial focus is in the board */
   gtk_widget_grab_focus (GTK_WIDGET (priv->board));
@@ -2433,9 +2459,6 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
 
   gobject_class->finalize = aisleriot_window_finalize;
 
-#ifndef HAVE_MAEMO
-  widget_class->configure_event = aisleriot_window_configure;
-#endif
   widget_class->window_state_event = aisleriot_window_state_event;
 
   g_type_class_add_private (gobject_class, sizeof (AisleriotWindowPrivate));
@@ -2516,21 +2539,22 @@ load_idle_cb (LoadIdleData *data)
                                      name);
     g_free (name);
 
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                              "%s", error->message);
-    g_error_free (error);
-
     if (priv->freecell_mode ||
-        (error->domain == g_quark_from_static_string ("AisleRiot") &&
-         error->code == GAME_ERROR_FALLBACK)) {
-      /* Loading the fallback game failed; all we can do is exit */
+        error->domain != AISLERIOT_GAME_ERROR ||
+        error->code != GAME_ERROR_FALLBACK) {
+      /* Loading freecell/the fallback game failed; all we can do is exit */
       g_signal_connect_swapped (dialog, "response",
                                 G_CALLBACK (gtk_widget_destroy), data->window);
     } else {
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+                                                "%s", error->message);
+
       g_signal_connect (dialog, "response",
                         G_CALLBACK (load_error_response_cb), data->window);
     }
 
+    g_error_free (error);
+
     gtk_window_present (GTK_WINDOW (dialog));
 
     return FALSE;
@@ -2540,7 +2564,7 @@ load_idle_cb (LoadIdleData *data)
    * store it in conf, except when we're running in freecell mode.
    */
   if (!priv->freecell_mode) {
-    aisleriot_conf_set_string (CONF_VARIATION, data->game_file);
+    games_conf_set_string (NULL, aisleriot_conf_get_key (CONF_VARIATION), data->game_file);
   }
 
   aisleriot_game_new_game (priv->game, data->seed != 0 ? &data->seed : NULL);
